<!DOCTYPE html>
<html lang="en">
<head>
    <title>{% block title %}{% endblock %} - Namer</title>
    <link href="https://cdnjs.cloudflare.com/ajax/libs/bootstrap/5.1.3/css/bootstrap.min.css" rel="stylesheet"/>
</head>
<body>
<div class="container">{% block content %}{% endblock %}</div>
<script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.6.0/jquery.min.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/bootstrap/5.1.3/js/bootstrap.bundle.min.js"></script>
<script src="{{ url_for('.static', filename='js/main.js') }}"></script>
</body>
</html>
