<div class="card h-100" style="width:12rem">
    <img class="card-img-top" src="{{ file['poster'] }}" alt="${value.title}">
    <div class="card-body">
        <h5 class="card-title">{{ file['title'] }}</h5>
        <p class="card-text">{{ file['date'] }}</p>
    </div>
    <div class="card-footer">
        <a href="https://metadataapi.net/scenes/{{ file['id'] }}" class="btn btn-secondary" target="_blank">Show</a>
        <button class="btn btn-primary float-end rename" data-bs-dismiss="modal" data-scene-id="{{ file['id'] }}" data-file="{{ data['file'] }}" onmouseup="window.location.reload(true)">
            Select
        </button>
    </div>
</div>
